@@ -22,8 +22,8 @@
 #include "config.h"
 #endif
 
-#undef LOCAL_DEBUG
-
+#define LOCAL_DEBUG
+/*#define DEBUG_SL2XIMAGE */
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -48,40 +48,12 @@
 # include <X11/extensions/XShm.h>
 #endif
 
-CARD32 color2pixel32bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel32rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel24bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel24rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel16bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel16rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel15bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel15rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
-CARD32 color2pixel_pseudo3bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
-CARD32 color2pixel_pseudo6bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
-CARD32 color2pixel_pseudo12bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
+#ifdef HAVE_GLX
+# include <GL/gl.h>
+# include <GL/glx.h>
+#endif
 
-void pixel2color32rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color32bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color24rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color24bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color16rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color16bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color15rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
-void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
 
-void ximage2scanline32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo6bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-
-void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage16( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage15( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo3bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo6bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
-void scanline2ximage_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data );
 
 #ifndef X_DISPLAY_MISSING
 static int  get_shifts (unsigned long mask);
@@ -95,14 +67,14 @@ asvisual_empty_XErrorHandler (Display * dpy, XErrorEvent * event)
     return 0;
 }
 /***************************************************************************/
-#ifdef LOCAL_DEBUG
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
 Status
 debug_AllocColor( const char *file, const char *func, int line, ASVisual *asv, Colormap cmap, XColor *pxcol )
 {
 	Status sret ;
 	sret = XAllocColor( asv->dpy, cmap, pxcol );
-	show_progress( " XAllocColor in %s:%s():%d has %s -> cmap = %lX, pixel = %d, color = 0x%4.4lX, 0x%4.4lX, 0x%4.4lX",
-				   file, func, line, (sret==0)?"failed":"succeeded", cmap, pxcol->pixel, pxcol->red, pxcol->green, pxcol->blue );
+	show_progress( " XAllocColor in %s:%s():%d has %s -> cmap = %lX, pixel = %lu(%8.8lX), color = 0x%4.4X, 0x%4.4X, 0x%4.4X",
+				   file, func, line, (sret==0)?"failed":"succeeded", cmap, pxcol->pixel, pxcol->pixel, pxcol->red, pxcol->green, pxcol->blue );
 	return sret;
 }
 #define ASV_ALLOC_COLOR(asv,cmap,pxcol)  debug_AllocColor(__FILE__, __FUNCTION__, __LINE__, (asv),(cmap),(pxcol))
@@ -174,7 +146,7 @@ prepare_scanline( unsigned int width, unsigned int shift, ASScanline *reusable_m
 	/* we want to align data by 8 byte boundary (double)
 	 * to allow for code with less ifs and easier MMX/3Dnow utilization :*/
 	aligned_width = width + (width&0x00000001);
-	sl->buffer = ptr = safemalloc (((aligned_width*4)+4)*sizeof(CARD32));
+	sl->buffer = ptr = safemalloc (((aligned_width*4)+16)*sizeof(CARD32));
 
 	sl->xc1 = sl->red 	= (CARD32*)(((long)ptr>>3)*8);
 	sl->xc2 = sl->green = sl->red   + aligned_width;
@@ -487,6 +459,15 @@ destroy_asvisual( ASVisual *asv, Bool reusable )
 					free( asv->as_colormap_reverse.xref );
 			}
 		}
+#ifdef HAVE_GLX
+		if( asv->glx_scratch_gc_direct )
+			glXDestroyContext(dpy, asv->glx_scratch_gc_direct );
+		if( asv->glx_scratch_gc_indirect )
+			glXDestroyContext(dpy, asv->glx_scratch_gc_indirect );
+#endif
+		if( asv->scratch_window ) 
+			XDestroyWindow( asv->dpy, asv->scratch_window );
+
 #endif /*ifndef X_DISPLAY_MISSING */
 		if( !reusable )
 			free( asv );
@@ -617,6 +598,39 @@ setup_truecolor_visual( ASVisual *asv )
 	if( vi->class != TrueColor )
 		return False;
 
+#ifdef HAVE_GLX
+	if( glXQueryExtension (dpy, NULL, NULL))
+	{
+		int val = False;
+		glXGetConfig(dpy, vi, GLX_USE_GL, &val);		
+		if( val ) 
+		{
+			asv->glx_scratch_gc_indirect = glXCreateContext (dpy, &(asv->visual_info), NULL, False);
+			if( asv->glx_scratch_gc_indirect ) 
+			{	
+				set_flags( asv->glx_support, ASGLX_Available );
+				if( glXGetConfig(dpy, vi, GLX_RGBA, &val) == 0 )
+					if( val ) set_flags( asv->glx_support, ASGLX_RGBA );
+				if( glXGetConfig(dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
+					if( val ) set_flags( asv->glx_support, ASGLX_DoubleBuffer );
+				if( glXGetConfig(dpy, vi, GLX_DOUBLEBUFFER, &val) == 0 )
+					if( val ) set_flags( asv->glx_support, ASGLX_DoubleBuffer );
+				
+				if( (asv->glx_scratch_gc_direct = glXCreateContext (dpy, &(asv->visual_info), NULL, True)) != NULL ) 
+					if( !glXIsDirect( dpy, asv->glx_scratch_gc_direct ) )
+					{	
+						glXDestroyContext(dpy, asv->glx_scratch_gc_direct );
+						asv->glx_scratch_gc_direct = NULL ;
+					}
+#if 0                          /* that needs some more research :  */
+				/* under Cygwin that seems to be 40% faster then regular XImage for some reason */
+				set_flags( asv->glx_support, ASGLX_UseForImageTx );
+#endif
+			}
+		}	 
+	}	 
+#endif
+
 	asv->BGR_mode = ((vi->red_mask&0x0010)!=0) ;
 	asv->rshift = get_shifts (vi->red_mask);
 	asv->gshift = get_shifts (vi->green_mask);
@@ -1026,13 +1040,11 @@ create_visual_gc( ASVisual *asv, Window root, unsigned long mask, XGCValues *gcv
 #ifndef X_DISPLAY_MISSING
 	if( asv )
 	{
-		Window scratch_window = 0 ;
 		XGCValues scratch_gcv ;
-		if( (scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL )) != None )
-		{
-			gc = XCreateGC( asv->dpy, scratch_window, mask, gcvalues?gcvalues:&scratch_gcv );
-			XDestroyWindow( asv->dpy, scratch_window );
-		}
+		if( asv->scratch_window == None ) 
+			asv->scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL );
+		if( asv->scratch_window != None )
+			gc = XCreateGC( asv->dpy, asv->scratch_window, gcvalues?mask:0, gcvalues?gcvalues:&scratch_gcv );
 	}
 #endif
 	return gc;
@@ -1044,7 +1056,13 @@ create_visual_pixmap( ASVisual *asv, Window root, unsigned int width, unsigned i
 #ifndef X_DISPLAY_MISSING
 	Pixmap p = None ;
 	if( asv != NULL )
-		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), (depth==0)?asv->true_depth:depth );
+	{	
+		if( root == None ) 
+			root = RootWindow(asv->dpy,DefaultScreen(asv->dpy));
+		if( depth==0 )
+			depth = asv->true_depth ;
+		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), depth );
+	}
 	return p;
 #else
 	return None ;
@@ -1180,6 +1198,10 @@ void remove_shm_area( ASShmArea *area, Bool free_resources )
 
 void flush_shm_cache( )
 {
+	if( xshmimage_images ) 
+		destroy_ashash( &xshmimage_images );
+	if( xshmimage_segments )
+		destroy_ashash( &xshmimage_segments );
 	while( shm_available_mem_head != NULL )
 		remove_shm_area( shm_available_mem_head, True );
 }
@@ -1231,7 +1253,7 @@ char *get_shm_area( int size, int *shmid )
 		return tmp ;
 	}
 
-	*shmid = shmget (IPC_PRIVATE, size, IPC_CREAT|0777);
+	*shmid = shmget (IPC_PRIVATE, size, IPC_CREAT|0666);
 	return shmat (*shmid, 0, 0);
 }
 
@@ -1241,7 +1263,7 @@ destroy_xshmimage_segment(ASHashableValue value, void *data)
 	ASXShmImage *img_data = (ASXShmImage*)data ;
 	if( img_data->segment != NULL )
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : segent to be freed: shminfo = %p, seg = %d ", img_data->segment, img_data->segment->shmid );
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : img_data = %p : segent to be freed: shminfo = %p ", img_data, img_data->segment );
 		XShmDetach (dpy, img_data->segment);
 		save_shm_area( img_data->segment->shmaddr, img_data->segment->shmid, img_data->size );
 		free( img_data->segment );
@@ -1250,7 +1272,7 @@ destroy_xshmimage_segment(ASHashableValue value, void *data)
 			free( img_data );
 	}else
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : segment data is NULL already value = %ld!!", value );
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : img_data = %p : segment data is NULL already value = %ld!!", img_data, value );
 	}
 }
 
@@ -1283,11 +1305,11 @@ destroy_xshmimage_image(ASHashableValue value, void *data)
 			orig_XShmImage_destroy_image( img_data->ximage );
 		else
 			XFree ((char *)img_data->ximage);
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_XIM : ximage freed: xim = %p", img_data->ximage);
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_XIM : ximage freed: img_data = %p, xim = %p", img_data, img_data->ximage);
 		img_data->ximage = NULL ;
 		if( img_data->segment != NULL && !img_data->wait_completion_event )
 		{
-			if( destroy_xshm_segment( img_data->segment->shmid ) )
+			if( destroy_xshm_segment( img_data->segment->shmseg ) )
 				return ;
 			img_data->segment = NULL ;
 		}
@@ -1344,9 +1366,10 @@ int destroy_xshm_image( XImage *ximage )
 unsigned long 
 ximage2shmseg( XImage *xim )
 {
-	ASXShmImage *data = NULL ;
-	if( get_hash_item( xshmimage_images, AS_HASHABLE(xim), &data ) == ASH_Success )		
+	void *vptr = NULL ;
+	if( get_hash_item( xshmimage_images, AS_HASHABLE(xim), &vptr ) == ASH_Success )		
 	{
+		ASXShmImage *data = (ASXShmImage *)vptr ;
 		if( data->segment ) 
 			return data->segment->shmseg;	
 	}	
@@ -1356,7 +1379,7 @@ ximage2shmseg( XImage *xim )
 void registerXShmImage( XImage *ximage, XShmSegmentInfo* shminfo )
 {
 	ASXShmImage *data = safecalloc( 1, sizeof(ASXShmImage));
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> CREATE_XIM : image created: xiom = %p, shminfo = %p, segment = %d, data = %p", ximage, shminfo, shminfo->shmid, ximage->data );
+	LOCAL_DEBUG_OUT( "XSHMIMAGE> CREATE_XIM : img_data = %p : image created: xiom = %p, shminfo = %p, segment = %d, data = %p", data, ximage, shminfo, shminfo->shmid, ximage->data );
 	data->ximage = ximage ;
 	data->segment = shminfo ;
 	data->size = ximage->bytes_per_line * ximage->height ;
@@ -1393,14 +1416,14 @@ Bool ASPutXImage( ASVisual *asv, Drawable d, GC gc, XImage *xim,
 
 	if( ( img_data = check_XImage_shared( xim )) != NULL )
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim );
+/*		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim ); */
 		if( XShmPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height, True ) )
 		{
 			img_data->wait_completion_event = True ;
 			return True ;
 		}
 	}
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim );
+/*	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim ); */
 	return XPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height );
 }
 
@@ -1412,7 +1435,7 @@ XImage *ASGetXImage( ASVisual *asv, Drawable d,
 
 	if( asv == NULL || d == None )
 		return NULL ;
-	if( _as_use_shm_images )
+	if( _as_use_shm_images && width*height > 4000)
 	{
 		unsigned int depth ;
 		Window        root;
@@ -1492,12 +1515,15 @@ create_visual_ximage( ASVisual *asv, unsigned int width, unsigned int height, un
 #if 0
 	unit = asv->dpy->bitmap_unit;
 #else
-	unit = (asv->true_depth+7)&0x0038;
+	if( depth == 0 ) 
+		unit = (asv->true_depth+7)&0x0038;
+	else
+		unit = (depth+7)&0x0038;
 	if( unit == 24 )
 		unit = 32 ;
 #endif
 #ifdef XSHMIMAGE
-	if( _as_use_shm_images )
+	if( _as_use_shm_images && width*height > 4000 )
 	{
 		XShmSegmentInfo *shminfo = safecalloc( 1, sizeof(XShmSegmentInfo));
 
@@ -1571,7 +1597,10 @@ create_visual_scratch_ximage( ASVisual *asv, unsigned int width, unsigned int he
 #if 0
 	unit = asv->dpy->bitmap_unit;
 #else
-	unit = (asv->true_depth+7)&0x0038;
+	if( depth == 0 )
+		unit = (asv->true_depth+7)&0x0038;
+	else
+		unit = (depth+7)&0x0038;
 	if( unit == 24 )
 		unit = 32 ;
 #endif
@@ -1756,6 +1785,7 @@ void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *g
 void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
+	register CARD32 *a = sl->alpha+sl->offset_x;
 	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
 	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
 	if( asv->msb_first )
@@ -1763,6 +1793,7 @@ void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regis
 		{
 			--i ;
 			{
+				a[i] = src[0];
 				r[i] = src[1];
 				g[i] = src[2];
 				b[i] = src[3];
@@ -1774,6 +1805,7 @@ void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regis
 		{
 			--i ;
 			{
+				a[i] = src[3];
 				r[i] = src[2];
 				g[i] = src[1];
 				b[i] = src[0];
@@ -1942,6 +1974,7 @@ ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,
 void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
+	register CARD32 *a = sl->alpha+sl->offset_x;
 	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
 	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
 	if( asv->msb_first )
@@ -1949,6 +1982,7 @@ void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 		{
 			--i ;
 			{
+				src[0] = a[i];
 				src[1] = r[i];
  				src[2] = g[i];
 				src[3] = b[i];
@@ -1960,6 +1994,7 @@ void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 		{
 			--i ;
 			{
+				src[3] = a[i];
 				src[2] = r[i];
 				src[1] = g[i];
 				src[0] = b[i];
