@@ -92,6 +92,9 @@
 #include <tiff.h>
 #include <tiffio.h>
 #endif
+#ifdef HAVE_SVG
+#include <librsvg/rsvg.h>
+#endif
 #ifdef HAVE_LIBXPM
 #ifdef HAVE_LIBXPM_X11
 #include <X11/xpm.h>
@@ -137,6 +140,7 @@ as_image_loader_func as_image_file_loaders[ASIT_Unknown] =
 	gif2ASImage ,
 	tiff2ASImage,
 	xml2ASImage ,
+	svg2ASImage ,
 	NULL,
 	tga2ASImage,
 	NULL,
@@ -160,6 +164,7 @@ const char *as_image_file_type_names[ASIT_Unknown+1] =
 	"GIF" ,
 	"TIFF",
 	"AfterStep XML" ,
+	"Scalable Vector Graphics (SVG)" ,
 	"XBM",
 	"Targa",
 	"PCX",
@@ -168,34 +173,22 @@ const char *as_image_file_type_names[ASIT_Unknown+1] =
 	"Unknown"
 };
 
- 
-ASImage *
-file2ASImage_extra( const char *file, ASImageImportParams *iparams )
+char *locate_image_file_in_path( const char *file, ASImageImportParams *iparams ) 
 {
 	int 		  filename_len ;
-	int 		  subimage = -1 ;
 	char 		 *realfilename = NULL, *tmp = NULL ;
 	register int i;
-
-	ASImage *im = NULL;
-	char *g_var ;
 	ASImageImportParams dummy_iparams = {0};
 
 	if( iparams == NULL )
 		iparams = &dummy_iparams ;
 
-	if( (g_var = getenv( "SCREEN_GAMMA" )) != NULL )
-		iparams->gamma = atof(g_var);
-
 	if( file )
 	{
 		filename_len = strlen(file);
 #ifdef _WIN32
-		{		
-			int i = 0 ; 
-			while( iparams->search_path[i] != NULL ) 
-				unix_path2dos_path( iparams->search_path[i] );
-		}
+		for( i = 0 ; iparams->search_path[i] != NULL ; ++i ) 
+			unix_path2dos_path( iparams->search_path[i] );
 #endif
 
 		/* first lets try to find file as it is */
@@ -222,7 +215,7 @@ file2ASImage_extra( const char *file, ASImageImportParams *iparams )
 			if( i < filename_len-1 && i > 0 )
 				if( tmp[i] == '.' )                 /* we have possible subimage number */
 				{
-					subimage = atoi( &tmp[i+1] );
+					iparams->subimage = atoi( &tmp[i+1] );
 					tmp[i] = '\0';
 					filename_len = i ;
 					realfilename = locate_image_file(tmp,iparams->search_path);
@@ -241,26 +234,64 @@ file2ASImage_extra( const char *file, ASImageImportParams *iparams )
 		if( tmp != realfilename && tmp != NULL )
 			free( tmp );
 	}
-	if( realfilename )
+	if( realfilename == file )
+		realfilename = mystrdup(file);
+	return realfilename ;
+}
+ASImage *
+file2ASImage_extra( const char *file, ASImageImportParams *iparams )
+{
+	char *realfilename ;
+	ASImage *im = NULL;
+	ASImageImportParams dummy_iparams = {0};
+	
+	if( iparams == NULL )
+		iparams = &dummy_iparams ;
+
+	realfilename = locate_image_file_in_path( file, iparams ); 
+	
+	if( realfilename != NULL ) 
 	{
 		ASImageFileTypes file_type = check_image_type( realfilename );
+
 		if( file_type == ASIT_Unknown )
 			show_error( "Hmm, I don't seem to know anything about format of the image file \"%s\"\n.\tPlease check the manual", realfilename );
 		else if( as_image_file_loaders[file_type] )
+		{
+			char *g_var = getenv( "SCREEN_GAMMA" );
+			if( g_var != NULL )
+				iparams->gamma = atof(g_var);
 			im = as_image_file_loaders[file_type](realfilename, iparams);
-		else
+		}else
 			show_error( "Support for the format of image file \"%s\" has not been implemented yet.", realfilename );
 #ifndef NO_DEBUG_OUTPUT
-		show_progress( "image loaded from \"%s\"", realfilename );
+		if( im != NULL ) 
+			show_progress( "image loaded from \"%s\"", realfilename );
 #endif
-		if( realfilename != file )
-			free( realfilename );
+		free( realfilename );
 	}else
 		show_error( "I'm terribly sorry, but image file \"%s\" is nowhere to be found.", file );
 
 	return im;
 }
 
+void init_asimage_import_params( ASImageImportParams *iparams ) 
+{
+	if( iparams ) 
+	{
+		iparams->flags = 0 ;
+		iparams->width = 0 ;
+		iparams->height = 0 ;
+		iparams->filter = SCL_DO_ALL ;
+		iparams->gamma = 0. ;
+		iparams->gamma_table = NULL ;
+		iparams->compression = 100 ;
+		iparams->format = ASA_ASImage ;
+		iparams->search_path = NULL;
+		iparams->subimage = 0 ;
+	}
+}
+
 ASImage *
 file2ASImage( const char *file, ASFlagType what, double gamma, unsigned int compression, ... )
 {
@@ -269,17 +300,15 @@ file2ASImage( const char *file, ASFlagType what, double gamma, unsigned int comp
 	ASImageImportParams iparams ;
 	va_list       ap;
 
-	iparams.flags = 0 ;
-	iparams.width = 0 ;
-	iparams.height = 0 ;
-	iparams.filter = SCL_DO_ALL ;
+	init_asimage_import_params( &iparams );
 	iparams.gamma = gamma ;
-	iparams.gamma_table = NULL ;
 	iparams.compression = compression ;
-	iparams.format = ASA_ASImage ;
 	iparams.search_path = &(paths[0]);
-	iparams.subimage = 0 ;
-
+#if 0
+	iparams.width = 1024 ; 
+	iparams.height = -1 ; 	
+	iparams.flags |= AS_IMPORT_SCALED_H|AS_IMPORT_SCALED_V ;
+#endif
 	va_start (ap, compression);
 	for( i = 0 ; i < MAX_SEARCH_PATHS ; i++ )
 	{	
@@ -333,6 +362,41 @@ file2pixmap(ASVisual *asv, Window root, const char *realfilename, Pixmap *mask_o
 	return trg ;
 }
 
+static ASImage *
+load_image_from_path( const char *file, char **path, double gamma)
+{
+	ASImageImportParams iparams ;
+
+	init_asimage_import_params( &iparams );
+	iparams.gamma = gamma ;
+	iparams.search_path = path;
+
+	return file2ASImage_extra( file, &iparams );
+}
+
+ASImageFileTypes
+get_asimage_file_type( ASImageManager* imageman, const char *file )
+{
+	ASImageFileTypes file_type = ASIT_Unknown ;
+	if( file )
+	{
+		ASImageImportParams iparams ;
+		char *realfilename ;
+	
+		init_asimage_import_params( &iparams );
+		iparams.search_path = imageman?&(imageman->search_path[0]):NULL;
+		realfilename = locate_image_file_in_path( file, &iparams ); 
+	
+		if( realfilename != NULL ) 
+		{
+			file_type = check_image_type( realfilename );
+			free( realfilename );
+		}
+	}
+	return file_type;
+}
+
+
 ASImage *
 get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsigned int compression )
 {
@@ -340,22 +404,178 @@ get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsign
 	if( imageman && file )
 		if( (im = fetch_asimage(imageman, file )) == NULL )
 		{
-			char **path = &(imageman->search_path[0]);
-			if( path[2] == NULL )
-				im = file2ASImage( file, what, imageman->gamma, compression, path[0], path[1], NULL );
-#if (MAX_SEARCH_PATHS!=8)
-#error "Fixme: please update file2ASImage call to match max number of search paths in ImageManager"
-#else
-			else
-				im = file2ASImage( file, what, imageman->gamma, compression, path[0], path[1], path[2],
-			    	               path[3], path[4], path[5], path[6], path[7], NULL );
-#endif
+			im = load_image_from_path( file, &(imageman->search_path[0]), imageman->gamma);
 			if( im )
+			{
 				store_asimage( imageman, im, file );
+				set_flags( im->flags, ASIM_NAME_IS_FILENAME );
+			}
+				
 		}
 	return im;
 }
 
+void 
+calculate_proportions( int src_w, int src_h, int *pdst_w, int *pdst_h ) 
+{
+	int dst_w = pdst_w?*pdst_w:0 ; 
+	int dst_h = pdst_h?*pdst_h:0 ; 
+	
+	if( src_w > 0 && src_w >= src_h && (dst_w > 0 || dst_h <= 0)) 
+		dst_h = (src_h*dst_w)/src_w ; 
+	else if( src_h > 0 ) 
+		dst_w = (src_w*dst_h)/src_h ; 
+
+	if( pdst_w ) *pdst_w = dst_w ; 	
+	if( pdst_h ) *pdst_h = dst_h ; 
+}
+
+ASImage *
+get_thumbnail_asimage( ASImageManager* imageman, const char *file, int thumb_width, int thumb_height, ASFlagType flags )
+{
+	ASImage *im = NULL ;
+	
+	if( imageman && file )
+	{
+#define AS_THUMBNAIL_NAME_FORMAT	"%s_scaled_to_%dx%d"
+		char *thumbnail_name = safemalloc( strlen(file)+sizeof(AS_THUMBNAIL_NAME_FORMAT)+32 );
+		ASImage *original_im = query_asimage(imageman, file );
+
+		if( thumb_width <= 0 && thumb_height <= 0 ) 
+		{
+			thumb_width = 48 ;
+			thumb_height = 48 ;
+		}
+
+		if( get_flags(flags, AS_THUMBNAIL_PROPORTIONAL ) ) 
+		{
+			if( original_im != NULL )		
+				calculate_proportions( original_im->width, original_im->height, &thumb_width, &thumb_height );
+		}else
+		{
+			if( thumb_width == 0 ) 
+				thumb_width = thumb_height ; 
+			if( thumb_height == 0 ) 
+				thumb_height = thumb_width ; 
+		}
+
+		if( thumb_width > 0 && thumb_height > 0 ) 
+		{
+			sprintf( thumbnail_name, AS_THUMBNAIL_NAME_FORMAT, file, thumb_width, thumb_height ) ;
+			im = fetch_asimage(imageman, thumbnail_name );
+			if( im == NULL )
+			{
+				if( original_im != NULL ) /* simply scale it down to a thumbnail size */
+				{
+					if( (( original_im->width > thumb_width || original_im->height > thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_REDUCE ) ) ||
+						(( original_im->width < thumb_width || original_im->height < thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) ) )
+					{
+						im = scale_asimage( NULL, original_im, thumb_width, thumb_height, ASA_ASImage, 100, ASIMAGE_QUALITY_FAST );
+						if( im != NULL ) 
+							store_asimage( imageman, im, thumbnail_name );
+					}else
+						im = dup_asimage( original_im );
+				}
+			}
+		}
+		
+		if( im == NULL ) 	
+		{
+			ASImage *tmp ; 
+			ASImageImportParams iparams ;
+
+			init_asimage_import_params( &iparams );
+			iparams.gamma = imageman->gamma ;
+			iparams.search_path = &(imageman->search_path[0]);
+			
+			iparams.width = thumb_width ; 
+			iparams.height = thumb_height ; 
+			iparams.flags |= AS_IMPORT_RESIZED|AS_IMPORT_SCALED_BOTH ; 
+			
+			if( get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) )
+				iparams.flags |= AS_IMPORT_FAST ; 
+			
+			tmp = file2ASImage_extra( file, &iparams );
+
+			if( tmp ) 
+			{
+				im = tmp ; 
+				if( tmp->width != thumb_width || tmp->height != thumb_height ) 
+				{
+					if( get_flags(flags, AS_THUMBNAIL_PROPORTIONAL ) ) 
+					{
+						calculate_proportions( tmp->width, tmp->height, &thumb_width, &thumb_height );
+						sprintf( thumbnail_name, AS_THUMBNAIL_NAME_FORMAT, file, thumb_width, thumb_height );
+						if( (im = query_asimage( imageman, thumbnail_name )) == NULL ) 
+							im = tmp ; 
+					}
+					if( im == tmp )
+					{
+						if( (( tmp->width > thumb_width || tmp->height > thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_REDUCE ) ) ||
+							(( tmp->width < thumb_width || tmp->height < thumb_height ) && !get_flags( flags, AS_THUMBNAIL_DONT_ENLARGE ) ) )
+						{
+							im = scale_asimage( NULL, tmp, thumb_width, thumb_height, ASA_ASImage, 100, ASIMAGE_QUALITY_FAST );
+							if( im == NULL ) 
+								im = tmp ;
+						}
+					}
+				}			
+				if( im != NULL )
+				{
+					if( im->imageman == NULL )
+						store_asimage( imageman, im, thumbnail_name );
+					else
+						dup_asimage( im );
+				}
+				
+				if( im != tmp ) 
+					destroy_asimage( &tmp );				
+			}
+		
+		}
+								 
+		if( thumbnail_name ) 
+			free( thumbnail_name );
+	}
+	return im;
+}
+
+
+Bool
+reload_asimage_manager( ASImageManager *imman )
+{
+#if (HAVE_AFTERBASE_FLAG==1)
+	if( imman != NULL ) 
+	{
+		ASHashIterator iter ;
+		if( start_hash_iteration (imman->image_hash, &iter) )
+		{
+			do
+			{
+				ASImage *im = curr_hash_data( &iter );
+/*fprintf( stderr, "im = %p. flags = 0x%lX\n", im, im->flags );		*/
+				if( get_flags( im->flags, ASIM_NAME_IS_FILENAME ) )
+				{
+/*fprintf( stderr, "reloading image \"%s\" ...", im->name );*/
+					ASImage *reloaded_im = load_image_from_path( im->name, &(imman->search_path[0]), imman->gamma);
+/*fprintf( stderr, "Done. reloaded_im = %p.\n", reloaded_im );*/					
+					if( reloaded_im ) 
+					{
+						if( asimage_replace (im, reloaded_im) ) 
+							free( reloaded_im );
+						else
+							destroy_asimage( &reloaded_im );
+					}				
+				}
+			}while( next_hash_item( &iter ) );
+			return True;		
+		}
+	}
+#endif
+	return False;
+}
+
+
 ASImageListEntry * 
 ref_asimage_list_entry( ASImageListEntry *entry )
 {
@@ -755,6 +975,8 @@ check_image_type( const char *realfilename )
 				type = ASIT_Targa;
 			else if (strncmp (&(head[0]), "#define", (size_t) 7) == 0)
 				type = ASIT_Xbm;
+			else if( mystrncasecmp(realfilename+filename_len-4, ".SVG", 4)==0 )
+				type = ASIT_SVG ;
 			else
 			{/* the nastiest check - for XML files : */
 				int i ;
@@ -770,7 +992,10 @@ check_image_type( const char *realfilename )
 					}
 					else if( head[i] != '<' )
 						type = ASIT_Unknown ;
-					else if( mystrncasecmp( &(head[i]), "<!DOCTYPE ", 10 ) == 0 ) 
+					else if( mystrncasecmp( &(head[i]), "<svg", 4 ) == 0 ) 
+					{
+						type = ASIT_SVG ;
+					}else if( mystrncasecmp( &(head[i]), "<!DOCTYPE ", 10 ) == 0 ) 
 					{	
 						type = ASIT_XML ;
 						for( i += 9 ; i < bytes_in ; ++i ) if( !isspace(head[i]) ) break;
@@ -815,6 +1040,14 @@ check_image_type( const char *realfilename )
 }
 
 
+ASImageFileTypes
+check_asimage_file_type( const char *realfilename )
+{
+	if( realfilename == NULL ) 
+		return ASIT_Unknown;
+	return check_image_type( realfilename );
+}
+
 /***********************************************************************************/
 #ifdef HAVE_XPM      /* XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM XPM */
 
@@ -1202,6 +1435,51 @@ jpeg2ASImage( const char * path, ASImageImportParams *params )
 	/* Adjust default decompression parameters */
 	cinfo.quantize_colors = FALSE;		       /* we don't want no stinking colormaps ! */
 	cinfo.output_gamma = params->gamma;
+	
+	if( get_flags( params->flags, AS_IMPORT_SCALED_BOTH ) == AS_IMPORT_SCALED_BOTH )
+	{
+		int w = params->width ; 
+		int h = params->height ;
+		int ratio ; 
+
+		if( w == 0 )
+		{
+			if( h == 0 ) 
+			{
+				w = cinfo.image_width ; 
+				h = cinfo.image_height ; 
+			}else
+				w = (cinfo.image_width * h)/cinfo.image_height ;
+		}else if( h == 0 )
+			h = (cinfo.image_height * w)/cinfo.image_width ;
+		
+		ratio = cinfo.image_height/h ; 
+		if( ratio > cinfo.image_width/w )
+			ratio = cinfo.image_width/w ; 
+		
+		cinfo.scale_num = 1 ; 
+		/* only supported values are 1, 2, 4, and 8 */
+		cinfo.scale_denom = 1 ; 
+		if( ratio >= 2 ) 
+		{
+			if( ratio >= 4 ) 
+			{
+				if( ratio >= 8 ) 
+					cinfo.scale_denom = 8 ; 
+				else
+					cinfo.scale_denom = 4 ; 
+			}else
+				cinfo.scale_denom = 2 ; 
+		}
+	}
+	
+	if( get_flags( params->flags, AS_IMPORT_FAST ) )
+	{/* this does not really makes much of a difference */
+		cinfo.do_fancy_upsampling = FALSE ; 
+		cinfo.do_block_smoothing = FALSE ; 
+		cinfo.dct_method = JDCT_IFAST ; 
+	}
+	
 	/* Step 5: Start decompressor */
 	(void)jpeg_start_decompress (&cinfo);
 	LOCAL_DEBUG_OUT("stored image size %dx%d", cinfo.output_width,  cinfo.output_height);
@@ -1738,13 +2016,17 @@ gif2ASImage( const char * path, ASImageImportParams *params )
 
 	START_TIME(started);
 
+	params->return_animation_delay = 0 ; 
+	
 	if ((fp = open_image_file(path)) == NULL)
 		return NULL;
 	if( (gif = open_gif_read(fp)) != NULL )
 	{
 		SavedImage	*sp = NULL ;
 		int count = 0 ;
-		if( (status = get_gif_saved_images(gif, params->subimage, &sp, &count )) == GIF_OK )
+		
+		status = get_gif_saved_images(gif, params->subimage, &sp, &count );
+		if( status == GIF_OK && sp != NULL && count > 0 )
 		{
 			GifPixelType *row_pointer ;
 #ifdef DEBUG_TRANSP_GIF
@@ -1756,13 +2038,32 @@ gif2ASImage( const char * path, ASImageImportParams *params )
 #ifdef DEBUG_TRANSP_GIF
 					fprintf( stderr, "%d: func = %X, bytes[0] = 0x%X\n", y, sp->ExtensionBlocks[y].Function, sp->ExtensionBlocks[y].Bytes[0]);
 #endif
-					if( sp->ExtensionBlocks[y].Function == 0xf9 &&
-			 			(sp->ExtensionBlocks[y].Bytes[0]&0x01))
+					if( sp->ExtensionBlocks[y].Function == GRAPHICS_EXT_FUNC_CODE ) 
 					{
-			   		 	transparent = ((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_GCE_TRANSPARENCY_BYTE])&0x00FF;
+						if( sp->ExtensionBlocks[y].Bytes[0]&0x01 )
+						{
+			   		 		transparent = ((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_GCE_TRANSPARENCY_BYTE])&0x00FF;
 #ifdef DEBUG_TRANSP_GIF
-						fprintf( stderr, "transp = %u\n", transparent );
+							fprintf( stderr, "transp = %u\n", transparent );
 #endif
+						}
+		   		 		params->return_animation_delay = (((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_GCE_DELAY_BYTE_LOW])&0x00FF) + 
+												   		((((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_GCE_DELAY_BYTE_HIGH])<<8)&0x00FF00);
+					}else if(  sp->ExtensionBlocks[y].Function == APPLICATION_EXT_FUNC_CODE && sp->ExtensionBlocks[y].ByteCount == 11 ) /* application extension */
+					{
+						if( strncmp(&(sp->ExtensionBlocks[y].Bytes[0]), "NETSCAPE2.0", 11 ) == 0 ) 
+						{
+							++y ;
+							if( y < (unsigned int)sp->ExtensionBlockCount && sp->ExtensionBlocks[y].ByteCount == 3 )
+							{
+				   		 		params->return_animation_repeats = (((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_NETSCAPE_REPEAT_BYTE_LOW])&0x00FF) + 
+														   		((((unsigned int) sp->ExtensionBlocks[y].Bytes[GIF_NETSCAPE_REPEAT_BYTE_HIGH])<<8)&0x00FF00);
+
+#ifdef DEBUG_TRANSP_GIF
+								fprintf( stderr, "animation_repeats = %d\n", params->return_animation_repeats );
+#endif
+							}
+						}
 					}
 				}
 			cmap = gif->SColorMap ;
@@ -1817,8 +2118,13 @@ gif2ASImage( const char * path, ASImageImportParams *params )
 				free(r);
 			}
 			free_gif_saved_images( sp, count );
-		}else
+		}else if( status != GIF_OK ) 
 			ASIM_PrintGifError();
+		else if( params->subimage == -1 )
+			show_error( "Image file \"%s\" does not have any valid image information.", path );
+		else
+			show_error( "Image file \"%s\" does not have subimage %d.", path, params->subimage );
+
 		DGifCloseFile(gif);
 		fclose( fp );
 	}
@@ -1858,7 +2164,11 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 
 	if( params->subimage > 0 )
 		if( !TIFFSetDirectory(tif, params->subimage))
-			show_warning("failed to read subimage %d from image file \"%s\". Reading first available instead.", params->subimage, path);
+		{
+			TIFFClose(tif);
+			show_error("Image file \"%s\" does not contain subimage %d.", path, params->subimage);
+			return NULL ;		
+		}
 
 	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &width);
 	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &height);
@@ -1875,6 +2185,7 @@ tiff2ASImage( const char * path, ASImageImportParams *params )
 		TIFFGetField(tif, TIFFTAG_TILELENGTH, &tile_length) )
 	{
 		show_error( "Tiled TIFF image format is not supported yet." );
+		TIFFClose(tif);
 		return NULL;   
 	}		
 
@@ -2008,6 +2319,108 @@ xml2ASImage( const char *path, ASImageImportParams *params )
 	SHOW_TIME("image loading",started);
 	return im ;
 }
+
+#ifdef HAVE_SVG/* SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG */
+ASImage *
+svg2ASImage( const char * path, ASImageImportParams *params )
+{
+   	static int gType_inited = 0;
+   
+   	ASImage *im = NULL;
+   	GdkPixbuf *pixbuf;
+	int channels ;
+	Bool do_alpha ; 
+	int width = -1, height = -1 ; 
+ 
+	START_TIME(started);
+
+	/* Damn gtk mess... must init once atleast.. can we just init
+	   several times or do we bork then? */
+	if (gType_inited == 0) 
+	{
+	   g_type_init();
+	   gType_inited = 1;
+	}
+ 
+ 	if( get_flags( params->flags, AS_IMPORT_SCALED_H ) )
+		width = (params->width <= 0)?((params->height<=0)?-1:params->height):params->width ;
+	
+ 	if( get_flags( params->flags, AS_IMPORT_SCALED_V ) )
+		height = (params->height <= 0)?((params->width <= 0)?-1:params->width):params->height ;
+		
+	if( (pixbuf = rsvg_pixbuf_from_file_at_size( path, width, height, NULL)) == NULL )
+		return NULL ;
+	
+	channels = gdk_pixbuf_get_n_channels(pixbuf) ;
+	do_alpha = gdk_pixbuf_get_has_alpha(pixbuf) ;
+	if ( ((channels == 4 && do_alpha) ||(channels == 3 && !do_alpha)) &&
+		gdk_pixbuf_get_bits_per_sample(pixbuf) == 8 ) 
+	{
+	   	int width, height;
+		register CARD8 *row = gdk_pixbuf_get_pixels(pixbuf);
+		int y;
+		CARD8 		 *r = NULL, *g = NULL, *b = NULL, *a = NULL ;
+
+		width = gdk_pixbuf_get_width(pixbuf);
+		height = gdk_pixbuf_get_height(pixbuf);
+
+		r = safemalloc( width );	   
+		g = safemalloc( width );	   
+		b = safemalloc( width );	   
+		if( do_alpha )
+			a = safemalloc( width );
+
+
+		im = create_asimage(width, height, params->compression );
+		for (y = 0; y < height; ++y) 
+		{
+			int x, i = 0 ;
+			for( x = 0 ; x < width ; ++x )
+			{
+				r[x] = row[i++];
+				g[x] = row[i++];
+				b[x] = row[i++];
+				if( do_alpha ) 
+					a[x] = row[i++];
+			}
+			im->channels[IC_RED][y]  = store_data( NULL, r, width, ASStorage_RLEDiffCompress, 0);
+		 	im->channels[IC_GREEN][y] = store_data( NULL, g, width, ASStorage_RLEDiffCompress, 0);	
+			im->channels[IC_BLUE][y]  = store_data( NULL, b, width, ASStorage_RLEDiffCompress, 0);
+
+			if( do_alpha )
+				for( x = 0 ; x < width ; ++x )
+					if( a[x] != 0x00FF )
+					{
+						im->channels[IC_ALPHA][y]  = store_data( NULL, a, width, ASStorage_RLEDiffCompress, 0);
+						break;
+					}
+			row += channels*width ;
+		}
+		free(r);
+		free(g);
+		free(b);
+		if( a )
+			free(a);
+	}
+	
+	if (pixbuf)
+		gdk_pixbuf_unref(pixbuf);
+	
+	SHOW_TIME("image loading",started);
+
+	return im ;
+}
+#else 			/* SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG */
+
+ASImage *
+svg2ASImage( const char * path, ASImageImportParams *params )
+{
+	show_error( "unable to load file \"%s\" - missing SVG image format libraries.\n", path );
+	return NULL ;
+}
+#endif			/* SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG SVG */
+
+
 /*************************************************************************/
 /* Targa Image format - some stuff borrowed from the GIMP.
  *************************************************************************/
