@@ -53,7 +53,11 @@ init_asfeel( ASFeel *feel )
     feel->ClickTime = 150;
     feel->AutoRaiseDelay = 0;
     feel->RaiseButtons = 0;
+#ifdef __CYGWIN__		
     feel->flags = DoHandlePageing;
+#else
+    feel->flags = DoHandlePageing/*|DontAnimateBackground*/;
+#endif
     feel->XorValue = (((unsigned long)1) << ASDefaultScr->d_depth) - 1;
 
     feel->no_snaping_mod = ShiftMask|ControlMask ;
@@ -79,7 +83,7 @@ create_asfeel()
 {
 	ASFeel *feel ;
 
-    feel = safemalloc( sizeof(ASFeel));
+    feel = safecalloc( 1, sizeof(ASFeel));
 	init_asfeel( feel );
 	return feel;
 }
@@ -138,6 +142,7 @@ destroy_asfeel( ASFeel *feel, Bool reusable )
                     destroy_aswindow_box( &(feel->window_boxes[i]), True );
                 free( feel->window_boxes );
             }
+			destroy_string( &(feel->default_window_box_name ) );
         }
 		if( !reusable )
 			free(feel);
@@ -278,8 +283,8 @@ LOCAL_DEBUG_CALLER_OUT( "feel %p", feel);
         {
             feel->window_boxes[i].area.height = ASDefaultScr->MyDisplayHeight ;
             if( get_flags( feel->window_boxes[i].flags, ASA_Virtual ) )
-                feel->window_boxes[i].area.width += ASDefaultScr->VyMax ;
-            feel->window_boxes[i].area.width -= feel->window_boxes[i].area.y ;
+                feel->window_boxes[i].area.height += ASDefaultScr->VyMax ;
+            feel->window_boxes[i].area.height -= feel->window_boxes[i].area.y ;
         }
         if( !get_flags(feel->window_boxes[i].flags, ASA_DesktopSet) )
             feel->window_boxes[i].desk = INVALID_DESK;
@@ -399,6 +404,24 @@ find_window_box( ASFeel *feel, const char *name )
 /*************************************************************************
  * Menus :
  *************************************************************************/
+void
+menu_data_destroy(ASHashableValue value, void *data)
+{
+    MenuData *md = data ;
+LOCAL_DEBUG_CALLER_OUT( "menu_data_destroy(\"%s\", %p)", (char*)value, data );
+    if( (char*)value )
+        free( (char*)value );
+    if( md )
+    {
+        if( md->magic == MAGIC_MENU_DATA )
+        {
+            if( md->name == (char*)value )
+                md->name = NULL ;
+			destroy_menu_data( &md ); 
+        }
+    }
+}
+
 void
 init_list_of_menus(ASHashTable **list, Bool force)
 {
